program moc

   implicit none
   
   integer, parameter                     :: ND=116, HH=4, NVARp=16, NVARo=2, PER=21*4, ITH=6
   integer                                :: i, j, nv, a, b, itemp
   integer, dimension(ITH)                :: iwork
   integer, dimension(NVARp)              :: icorrel

   double precision                       :: mediax, mediay, varx, vary, rth=0.4
   double precision                       :: covarxy, temp, yest
   double precision, dimension(ND)        :: wspdobs
   double precision, dimension(PER)       :: x, y
   double precision, dimension(NVARp)     :: correlxy, correlord
   double precision, dimension(ND, NVARp) :: prev
   double precision, dimension(ND, NVARo) :: obs
   double precision, dimension(PER, ITH)  :: xmr
   
   double precision, dimension(PER+2*ITH) :: dwork

   character(len=5), dimension(NVARp)     :: varp
   character(len=5), dimension(NVARo)     :: varo
   
   

!  Input data: previsao -----------------------------------------------   
   open(10, file='../datain/previsao.txt')

   do j=1, NVARp-1
      read(10, *) varp(j), (prev(i, j), i=1, ND)
      icorrel(j)=j
   enddo
   prev(:, NVARp)=sqrt(prev(:, 1)**2 + prev(:, 2)**2)
   varp(NVARp)="intv"
   icorrel(NVARp)=NVARp
  
   
!  observacao: -------------------------------------------------------   
   open(11, file='../datain/obs.txt')
   
   do j=1, NVARo
      read(11, *) varo(j), (obs(i, j), i=1, ND)
   enddo
   wspdobs=sqrt(obs(:, 1)**2 + obs(:, 2)**2)
      

!--------------------------------------------------------------------------------------------
!  Esquema de armazenamento das variaveis:
!  
!  prev(ND, NVARp) ::  ND    :: numero de horarios dos dias corridos
!                          NVARp :: numero de variaveis: 1->10u, 2->10v, ..., 15->ultimavar
!
!--------------------------------------------------------------------------------------------


!--------------------------------------------------------------------------------------------
!  1a tentativa: calcular u e v separadamente 
!  2a          : calcular o vento u;v
!XX3a          : calcular para os horarios separados: 00, 00, ...  deu ruim (correl baixa)
!  4a          : calcular para os horarios corridos: 00, 06, 12, ...


!  X :: vars independentes y(x)=x1...  previsoes
!  Y :: var dependente                 obs 
!--------------------------------------------------------------------------------------------

  correlxy=0.0
  do nv=1, NVARp

     y=wspdobs(1:PER)
     x=prev(1:PER, nv)



!     -------------------------
!     media:   
!
      mediax = sum(x)/PER
      mediay = sum(y)/PER





!     -------------------------
!     variancia:   
!
      varx = sum((x - mediax) * (x - mediax))
      vary = sum((y - mediay) * (y - mediay))




!     -------------------------
!     covariancia:   
!
      covarxy = sum((x - mediax) * (y - mediay))




!     -------------------------
!     correlacao:   
!
      correlxy(nv)=covarxy/sqrt(varx)/sqrt(vary)
   

!      open(20, file='../dataout/serie.bin', access='direct', form='unformatted', status='unknown', recl=2*4)
!
!      do i=1, ND
!         print*, i, y(i), x(i)
!         write(20, rec=i) prev(i, 1), obs(i, 1)
!      enddo
   
   
   enddo   
   
      print*,''
   
!  Ordenando o vetor de correlacoes:
   correlord=abs(correlxy)
   
!   do i=1, NVARp
!      print*, i, correlxy(i), correlord(i), icorrel(i)
!   enddo

   do a=1, NVARp
      do b=a, NVARp
         if (correlord(a) <= correlord(b))then
            temp=correlord(a)
            itemp=icorrel(a)

            correlord(a)=correlord(b)
            icorrel(a)=icorrel(b)

            correlord(b)=temp
            icorrel(b)=itemp
         endif
      enddo
   enddo 
  
   do i=1, NVARp
      print*, icorrel(i), varp(icorrel(i)), correlord(i)
   enddo


!  Separando as 6 (ITH) variaveis escolhidas para Reg. Multipla:

   do i=1, ITH
      xmr(:, i)=prev(1:PER, icorrel(i))
   enddo   

   CALL MR (xmr, y, PER, ITH, DWORK, IWORK)



!  Minha nova estimativa: WSPD_estimado(PER+1)
!   yest=B1*x1+B2*x2+...+B6*X6 (t=PER+1)
   
   yest=0.0
   do i=1, ITH
      yest=yest+y(i)*prev(PER+1, icorrel(i))
   enddo

   write(*,"(' WSPD_Prev: ', f20.4)") prev(PER+1, 16)
   write(*,"(' WSPD_Obs : ', f20.4)") wspdobs(PER+1)
   write(*,"(' WSPD_Est : ', f20.4)") yest



! Proximos passos:
!=== - definir o TH (por num.var, ou por n.corr)
!=== - definir o corte do periodo a ser treinado
!=== - implementar rotina RegMult.(MR)
!=== - usar a rotina MR
!=== - aplicar valores t+1 na funcao
!=== - calcular a funcao(t+1)
!=== - avaliar!
 
 
 
 
 
 
      
   

end program moc



!-----------------------------------------------------------------------
! MR - multiple regression using the SLATEC library routine DHFTI
!
! Finds the nearest approximation to BETA in the system of linear equations:
!                     
!              X(j,i) . BETA(i) = Y(j)
! where   
!                  1 ... j ... N  
!                  1 ... i ... K
! and               
!                  K .LE. N
!
! INPUT ARRAYS ARE DESTROYED!
!
!___Name___________Type_______________In/Out____Description_____________
!   X(N,K)         Double precision   In        Predictors
!   Y(N)           Double precision   Both      On input:   N Observations
!                                               On output:  K beta weights
!   N              Integer            In        Number of observations
!   K              Integer            In        Number of predictor variables
!   DWORK(N+2*K)   Double precision   Neither   Workspace
!   IWORK(K)       Integer            Neither   Workspace
!-----------------------------------------------------------------------
      SUBROUTINE MR (X, Y, N, K, DWORK, IWORK)
       IMPLICIT NONE
       INTEGER K, N, IWORK
       DOUBLE PRECISION X, Y, DWORK
       DIMENSION X(N,K), Y(N), DWORK(N+2*K), IWORK(K)
       
!         local variables
       INTEGER I, J
       DOUBLE PRECISION TAU, TOT
       
!        maximum of all column sums of magnitudes
       TAU = 0.
       DO J = 1, K
         TOT = 0.
         DO I = 1, N
           TOT = TOT + ABS(X(I,J))
         END DO
         IF (TOT > TAU) TAU = TOT
       END DO
       TAU = TAU * EPSILON(TAU)        ! tolerance argument
       
!            call function
       CALL DHFTI (X, N, N, K, Y, N, 1, TAU, J, DWORK(1), DWORK(N+1), DWORK(N+K+1), IWORK)
       IF (J < K) PRINT *, 'mr: solution is rank deficient!', J
       RETURN
      END  ! of MR


