program moc

   implicit none
   
   integer, parameter                     :: ND=109, HH=4, NVAR=15, NVARp=9, NVARo=2, PER=10*4, ITH=6
   integer                                :: nv, nt, i, j, a, b, itemp
   integer, dimension(ITH)                :: iwork
   integer, dimension(NVARp)              :: icorrel

   double precision                       :: mediax, mediay, varx, vary, rth=0.4
   double precision                       :: covarxy, temp, yest, lat, lon
   double precision, dimension(PER)       :: x, y
   double precision, dimension(NVARp)     :: correlxy, correlord
   double precision, dimension(ND, NVARp) :: prev
   double precision, dimension(ND)        :: obs
   double precision, dimension(PER, ITH)  :: xmr
   
   double precision, dimension(PER+2*ITH) :: dwork

   character(len=10)                       :: lix
   character(len=10), dimension(ND)        :: yyyymmddhh
   character(len=10), dimension(NVARp)     :: varp
   character(len=10)                       :: varo
   
   

!  Input data:  -----------------------------------------------   
   open(10, file='../datain/A433.txt')
   read(10, *) (lix, i=1, 5), varo, (varp(i),  i=1, NVARp)
   
   do nt=1, ND
      read(10, *) yyyymmddhh(nt), lat, lon, lat, lon, obs(nt), (prev(nt, nv), nv=1, NVARp)
   enddo

   correlxy=0.0
   do nv=1, NVARp
      icorrel(nv)=nv

      y=obs(1:PER)
      x=prev(1:PER, nv)
      
!     -------------------------
!     media:   
!
      mediax = sum(x)/PER
      mediay = sum(y)/PER





!     -------------------------
!     variancia:   
!
      varx = sum((x - mediax) * (x - mediax))
      vary = sum((y - mediay) * (y - mediay))




!     -------------------------
!     covariancia:   
!
      covarxy = sum((x - mediax) * (y - mediay))




!     -------------------------
!     correlacao:   
!
      correlxy(nv)=covarxy/sqrt(varx)/sqrt(vary)
   enddo

!  Ordenando o vetor de correlacoes:
   correlord=abs(correlxy)


   do a=1, NVARp
      do b=a, NVARp
         if (correlord(a) <= correlord(b))then
            temp=correlord(a)
            itemp=icorrel(a)

            correlord(a)=correlord(b)
            icorrel(a)=icorrel(b)

            correlord(b)=temp
            icorrel(b)=itemp
         endif
      enddo
   enddo 

!   do i=1, NVARp
!      write(*, '(i2, 2x, a, 2x, f5.2)') icorrel(i), varp(icorrel(i)), correlord(i)
!   enddo

!  Separando as 6 (ITH) variaveis escolhidas para Reg. Multipla:

   do i=1, ITH
      xmr(:, i)=prev(1:PER, icorrel(i))
      write(*, '(i2, 1x, i2, 2x, a, 2x, f5.2)') i, icorrel(i), varp(icorrel(i)), correlord(i)
   enddo   

   CALL MR (xmr, y, PER, ITH, DWORK, IWORK)

!  Minha nova estimativa: WSPD_estimado(PER+1)
!  yest=B1*x1+B2*x2+...+B6*X6 (t=PER+1)


   print*, ''
   do j=1, 4
      yest=0.0
      do i=1, ITH
         yest=yest+y(i)*prev(PER+j, icorrel(i))
      enddo
      print*, yyyymmddhh(PER+j)
      write(*,"(' WSPD_Prev: ', f20.4)") prev(PER+j, 1)
      write(*,"(' WSPD_Obs : ', f20.4)") obs(PER+j)
      write(*,"(' WSPD_Est : ', f20.4)") yest
      print*, ''
   enddo




end




!-----------------------------------------------------------------------
! MR - multiple regression using the SLATEC library routine DHFTI
!
! Finds the nearest approximation to BETA in the system of linear equations:
!                     
!              X(j,i) . BETA(i) = Y(j)
! where   
!                  1 ... j ... N  
!                  1 ... i ... K
! and               
!                  K .LE. N
!
! INPUT ARRAYS ARE DESTROYED!
!
!___Name___________Type_______________In/Out____Description_____________
!   X(N,K)         Double precision   In        Predictors
!   Y(N)           Double precision   Both      On input:   N Observations
!                                               On output:  K beta weights
!   N              Integer            In        Number of observations
!   K              Integer            In        Number of predictor variables
!   DWORK(N+2*K)   Double precision   Neither   Workspace
!   IWORK(K)       Integer            Neither   Workspace
!-----------------------------------------------------------------------
      SUBROUTINE MR (X, Y, N, K, DWORK, IWORK)
       IMPLICIT NONE
       INTEGER K, N, IWORK
       DOUBLE PRECISION X, Y, DWORK
       DIMENSION X(N,K), Y(N), DWORK(N+2*K), IWORK(K)
       
!         local variables
       INTEGER I, J
       DOUBLE PRECISION TAU, TOT
       
!        maximum of all column sums of magnitudes
       TAU = 0.
       DO J = 1, K
         TOT = 0.
         DO I = 1, N
           TOT = TOT + ABS(X(I,J))
         END DO
         IF (TOT > TAU) TAU = TOT
       END DO
       TAU = TAU * EPSILON(TAU)        ! tolerance argument
       
!            call function
       CALL DHFTI (X, N, N, K, Y, N, 1, TAU, J, DWORK(1), DWORK(N+1), DWORK(N+K+1), IWORK)
       IF (J < K) PRINT *, 'mr: solution is rank deficient!', J
       RETURN
      END  ! of MR



